#include "ParticleSystem.h"
#include "RenderingHelper.h"

#include <cmath>
#include <iostream>

#ifndef M_PI_4
	#define M_PI_4 (3.14159265358979323846f / 4)
#endif

#ifndef M_SQRT2
	#define M_SQRT2 1.41421356237309504880f
#endif

void ParticleSystem::Update(float dt)
{
	EmitersIterator it = emiters.begin();

	while (it != emiters.end())
	{
		if ((*it)->Update(this, dt))
		{
			it++;
		}
		else
		{
			emiters.erase(it++);
		}
	}

	Particle* p = firstParticle;

	while (p)
	{
		p->changeDuration -= dt;

		// czsteczka w trakcie zmiany
		if (p->changeDuration >= 0)
		{
			// aktualizacja parametrw
			p->position += p->velocity * dt;
			
			p->current.color += p->change.color * dt;
			p->current.angle += p->change.angle * dt;
			p->current.size += p->change.size * dt;

			p = p->nextParticle;
		}
		else
		{
			// zniszczenie czsteczki
			Particle* toDestroy = p;
			
			p = p->nextParticle;

			DestroyParticle(toDestroy);
		}
	}
}


void ParticleSystem::Render()
{
	RenderingHelper* helper = RenderingHelper::GetInstance();

	Particle* p = firstParticle;

	if (texture) 
		texture->SetAsCurrent();
	else
		glBindTexture(GL_TEXTURE_2D, 0);

	helper->SetBlendMode(blendMode);

	helper->BeginRendering();

	while (p)
	{
		float halfSize = p->current.size / 2.0f;

		Vertex* quad = helper->GetQuad();

		float angle = p->current.angle - M_PI_4;
		
		float x = halfSize * M_SQRT2 * cos(angle);
		float y = halfSize * M_SQRT2 * sin(angle);

		quad[0].x = p->position.x + x;
		quad[0].y = p->position.y + y;
		quad[0].u = p->uvRegion->u0;
		quad[0].v = p->uvRegion->v0;
		quad[0].color = p->current.color;

		quad[1].x = p->position.x + y;
		quad[1].y = p->position.y - x;
		quad[1].u = p->uvRegion->u1;
		quad[1].v = p->uvRegion->v0;
		quad[1].color = p->current.color;

		quad[2].x = p->position.x - x;
		quad[2].y = p->position.y - y;
		quad[2].u = p->uvRegion->u1;
		quad[2].v = p->uvRegion->v1;
		quad[2].color = p->current.color;

		quad[3].x = p->position.x - y;
		quad[3].y = p->position.y + x;
		quad[3].u = p->uvRegion->u0;
		quad[3].v = p->uvRegion->v1;
		quad[3].color = p->current.color;

		p = p->nextParticle;
	}

	helper->EndRendering();
}