#include "RenderingHelper.h"

#include <SDL/SDL_opengl.h>

RenderingHelper* RenderingHelper::instance = 0;

RenderingHelper::RenderingHelper() : nVertices(0), nIndices(0)
{
	for (int i = 0; i < MAX_VERTICES; i++) vertices[i].z = 0;

	memset(debugPrints, 1, sizeof(debugPrints));

	debugPrintFont = Texture::Load("debug_font.png");
}

void RenderingHelper::BeginRendering()
{
	glVertexPointer(3, GL_FLOAT, sizeof(Vertex), &vertices->x);
	glColorPointer(4, GL_FLOAT, sizeof(Vertex), &vertices->color);
	glTexCoordPointer(2, GL_FLOAT, sizeof(Vertex), &vertices->u);
}

void RenderingHelper::EndRendering()
{
	Flush();
}

void RenderingHelper::Flush()
{
	if (nVertices == 0) return;

	glDrawElements(GL_TRIANGLES, nIndices, GL_UNSIGNED_SHORT, indices);

	nVertices = 0;
	nIndices = 0;
}

void RenderingHelper::SetBlendMode(BlendMode blendMode)
{
	switch (blendMode)
	{
	case BlendModes::Add:
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		break;
	case BlendModes::Disabled:
		glDisable(GL_BLEND);
		break;
	default:		
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		break;
	}
}


void RenderingHelper::DrawDebugInformation()
{
	BeginRendering();

	SetBlendMode(BlendModes::Normal);

	debugPrintFont->SetAsCurrent();

	float uvStep = 1 / 16.0f;

	for (int y = 0; y < DEBUG_H; y++)
	{
		for (int x = 0; x < DEBUG_W; x++)
		{
			char c = debugPrints[y][x];

			if (c == 0) break;
			if (c < 32) continue;

			int cx = c % 16;
			int cy = c / 16;
			
			Vertex* quad = GetQuad();

			quad[0].x = x * 16.0f;
			quad[0].y = y * 16.0f;
			quad[0].u = cx * uvStep;
			quad[0].v = cy * uvStep;
			quad[0].color = Color(1, 1, 1);

			quad[1].x = (x + 1) * 16.0f;
			quad[1].y = y * 16.0f;
			quad[1].u = (cx + 1) * uvStep;
			quad[1].v = cy * uvStep;
			quad[1].color = Color(1, 1, 1);

			quad[2].x = (x + 1) * 16.0f;
			quad[2].y = (y + 1) * 16.0f;
			quad[2].u = (cx + 1) * uvStep;
			quad[2].v = (cy + 1) * uvStep;
			quad[2].color = Color(1, 1, 1);

			quad[3].x = x * 16.0f;
			quad[3].y = (y + 1) * 16.0f;
			quad[3].u = cx * uvStep;
			quad[3].v = (cy + 1) * uvStep;
			quad[3].color = Color(1, 1, 1);
		}
	}

	EndRendering();
}

void RenderingHelper::WriteDebugLine(int y, const char *format, ...)
{
	if (y < 0 || y >= DEBUG_H) return;

	va_list ap;
	va_start(ap, format);

	vsnprintf(debugPrints[y], DEBUG_W, format, ap);

	va_end(ap);
}

void RenderingHelper::SetClearColor(const Color& color)
{
	glClearColor(color.r, color.g, color.b, color.a);
}
