#ifndef RENDERING_HELPER_H
#define RENDERING_HELPER_H

#include "Color.h"
#include "Texture.h"

namespace BlendModes
{
	enum BlendMode
	{
		Disabled,
		Normal,
		Add,
		Count
	};
}

typedef BlendModes::BlendMode BlendMode;

struct Vertex
{
	float x, y, z;
	
    Color color;

	float u, v;
};


typedef unsigned short u16;

#define MAX_VERTICES 2048

#define DEBUG_W 40
#define DEBUG_H 30

class RenderingHelper
{
	RenderingHelper();
	static RenderingHelper* instance;

	Vertex vertices[MAX_VERTICES];

	// na kade 4 wierzchoki przypada 6 indeksw
	u16 indices[(3 * MAX_VERTICES) / 2];

	int nVertices;
	int nIndices;

	char debugPrints[DEBUG_H][DEBUG_W];
	TextureRef debugPrintFont;
public:
	static RenderingHelper* GetInstance()
	{
		if (!instance) instance = new RenderingHelper();
		return instance;
	}

	void BeginRendering();
	void EndRendering();
	void Flush();

	Vertex* GetQuad()
	{
		if (nVertices > MAX_VERTICES - 4) Flush();

		Vertex* quad = vertices + nVertices;

		u16* idx = indices + nIndices;

		idx[0] = nVertices;
		idx[1] = nVertices + 1;
		idx[3] = nVertices + 2;
		idx[2] = nVertices + 2;
		idx[4] = nVertices + 3;
		idx[5] = nVertices;

		nVertices += 4;
		nIndices += 6;

		return quad;
	}

	void SetBlendMode(BlendMode blendMode);

	void DrawDebugInformation();
	void WriteDebugLine(int y, const char *format, ...);

	void SetClearColor(const Color& color);
};

#endif