#include "Texture.h"
#include <SDL/SDL_image.h>

#include <iostream>
using std::cout;
using std::endl;

TextureRef Texture::Load(const char* fileName)
{
	return TextureRef(new Texture(fileName));
}

bool Texture::Initialize(SDL_Surface* surface)
{
	if (((surface->w & (surface->w - 1)) != 0) || ((surface->h & (surface->h - 1)) != 0))
	{
		cout << "Bd: Wymiary tekstury nie s potg liczby 2." << endl;
		return false;
	}

	width = surface->w;
	height = surface->h;

	int nColors = surface->format->BytesPerPixel;
	GLenum textureFormat;

	if (nColors == 4)
	{
		textureFormat = (surface->format->Rmask == 0x000000ff) ? GL_RGBA : GL_BGRA;
	} 
	else if (nColors == 3)
	{
		textureFormat = (surface->format->Rmask == 0x000000ff) ? GL_RGB : GL_BGR;
	}
	else
	{
		cout << "Bd: Niepoprawny format tekstury." << endl;
		return false;
	}

	glGenTextures(1, &textureId);
	glBindTexture(GL_TEXTURE_2D, textureId);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, nColors, width, height, 0,
				textureFormat, GL_UNSIGNED_BYTE, surface->pixels );

	return true;
}


Texture::Texture(const char* fileName)
{
	cout << "Wczytywanie tekstury: " << fileName << endl;

	SDL_Surface* surface = IMG_Load(fileName);

	if (!surface)
	{
		cout << "Bd: Wczytywanie nie powido si." << endl;
		width = height = textureId = 0;
		return;
	}

	if (!Initialize(surface))
	{
		width = height = textureId = 0;
	}

	SDL_FreeSurface(surface);
}

void Texture::SetAsCurrent()
{
	glBindTexture(GL_TEXTURE_2D, textureId);
}

Texture::~Texture()
{
	glDeleteTextures(1, &textureId);

	ReleaseNamedRegions();
}

using std::string;
using std::map;

const UVRegion DefaultRegion(0, 0, 1, 1);

// ustawienie parametrw regionu
const UVRegion* Texture::SetNamedRegion(const string& name, float u0, float v0, float u1, float v1)
{
	map<string, UVRegion*>::iterator it = regions.find(name);

	if (it != regions.end())
	{	
		// region jeszcze nie istnieje
		UVRegion* region = it->second;

		region->u0 = u0;
		region->v0 = v0;
		region->u1 = u1;
		region->v1 = v1;

		return region;
	}
	else
	{
		// modyfikacja istniejcego regionu
		UVRegion* region = new UVRegion(u0, v0, u1, v1);

		regions.insert(make_pair(name, region));

		return region;
	}
}

// pobranie regionu
const UVRegion* Texture::GetNamedRegion(const string& name)
{
	map<string, UVRegion*>::iterator it = regions.find(name);

	if (it != regions.end())
	{
		// region istnieje
		return it->second;
	}
	else
	{
		// region nie istnieje -- zwracamy region obejmujcy
		// ca tekstur
		return &DefaultRegion;
	}
}

// zwolnienie wszystkich regionw
void Texture::ReleaseNamedRegions()
{
	map<string, UVRegion*>::iterator it;

	for (it = regions.begin(); it != regions.end(); it++)
	{
		delete it->second;
	}

	regions.clear();
}